<?php

require_once('api/Simpla.php');
require_once(__DIR__ . '/include.php');

class RBSPayment extends Simpla
{

    public $getaway_url;
    public $module_version = '1.2.0';
    public $allowCallbacks = RBS_ENABLE_CALLBACK;

    public function checkout_form($order_id, $button_text = null)
    {
        if ($this->request->method('post') && $this->request->post('go')) {
            $this->redirect($order_id);
        } else {
            $button = "<form method=POST>" .
                "<input name=go type=submit class=checkout_button
						 value='Перейти к оплате &#8594;'>" .
                "</form>";
            return $button;
        }
    }

    public function redirect($order_id)
    {
        $order = $this->orders->get_order((int)$order_id);

        $payment_method = $this->payment->get_payment_method($order->payment_method_id);
        $payment_settings = $this->payment->get_payment_settings($payment_method->id);

        $price = $this->money->convert($order->total_price, $payment_method->currency_id, false);

        $return_url = $this->config->root_url . '/payment/RBSPayment/callback.php?o=' . $order->id;

        $action_adr = RBS_TEST_URL;
        if ($payment_settings['test_mode'] != 1) {
            $action_adr = RBS_PROD_URL;
            // for migrate to new payment gateway
            if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
                if (substr($this->merchant, 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                    $action_adr = RBS_PROD_URL_ALT;
                } else {
                    //disable callbacks for old gateway version
                    $this->allowCallbacks = false;
                }
            }
        }

        if ($payment_settings['stage_mode'] == 'two-stage') {
            $action_adr .= 'registerPreAuth.do';
        } else {
            $action_adr .= 'register.do';
        }

        $args = array(
            'userName' => $payment_settings['merchant'],
            'password' => $payment_settings['password'],
            'orderNumber' => $order->id . '_' . time(),
            'amount' => $price * 100,
            'returnUrl' => $return_url,
            'jsonParams' => json_encode(
                array(
                    'CMS: ' => 'SimplaCMS version: ' . $this->config->version,
                    'Module-Version: ' => $this->module_version
                )
            ),
        );

        // ------------------------------------------------------------------------------------------------
        //ob
        if (RBS_ENABLE_FISCALE_OPTIONS === true && $payment_settings['send_order'] == 1) {

            $args['amount'] = 0; //PLUG-3123 calculate amont inside orderBundle
            $args['taxSystem'] = $payment_settings['tax_system'];

            $order_items = $this->orders->get_purchases(array('order_id' => $order->id));

            $order_date_created = strtotime($order->date);
            $order_billing_email = $order->email;
            $order_billing_phone = $order->phone;

            $items = array();
            $itemsCnt = 1;


            /* Заполнение массива данных корзины */
            foreach ($order_items as $value) {
                $item = array();

                $product_price = round(($value->price) * 100);

                $item['positionId'] = $itemsCnt++;
                $item['name'] = $value->product_name;

                $item['quantity'] = array(
                    'value' => $value->amount,
                    'measure' => ($payment_settings['FFDVersion'] == 'v1_05') ? RBS_DEFAULT_MEASURE_NAME : RBS_MEASUREMENT_CODE
                );
                $item['itemAmount'] = $product_price * $value->amount;
                $item['itemCode'] = $value->product_id . "_" . $item['positionId'];
                $item['tax'] = array(
                    'taxType' => 0 // <--------------------
                );
                $item['itemPrice'] = $product_price;

                $args['amount'] += $item['itemAmount'];

                // FFD 1.05 added
//                if ($payment_settings['FFDVersion'] == 'v1_05') {

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $payment_settings['ffd_paymentMethodType']
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => $payment_settings['ffd_paymentObjectType']
                );

                $item['itemAttributes']['attributes'] = $attributes;
//                }

                $items[] = $item;
            }

            // DELIVERY POSITION
            if ($order->delivery_price > 0) {
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = 'Доставка';
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => ($payment_settings['FFDVersion'] == 'v1_05') ? RBS_DEFAULT_MEASURE_NAME : RBS_MEASUREMENT_CODE
                );
                $itemShipment['itemAmount'] = $itemShipment['itemPrice'] = $order->delivery_price * 100;

                $args['amount'] += $itemShipment['itemAmount'];

                $itemShipment['itemCode'] = 'Delivery';
                $itemShipment['tax'] = array(
                    'taxType' => 0 // <--------------------
                );

                // FFD 1.05 added
//                if ($payment_settings['FFDVersion'] == 'v1_05') {
                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $payment_settings['ffd_paymentMethodType_Delivery']
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );

                $itemShipment['itemAttributes']['attributes'] = $attributes;

//                }

                $items[] = $itemShipment;
            }


            /* Создание и заполнение массива данных заказа для фискализации */
            $order_bundle = array(
                'orderCreationDate' => $order_date_created,
                'customerDetails' => array(
                    'email' => $order_billing_email
                ),
                'cartItems' => array('items' => $items)
            );

            /* Заполнение массива данных для запроса c фискализацией */
            $args['orderBundle'] = json_encode($order_bundle);
        }
        // ------------------------------------------------------------------------------------------------

        if ($this->allowCallbacks) {

            if ($payment_settings['test_mode'] == 1) {
                $gate_url = str_replace("payment/rest", "mportal-uat/mvc/public/merchant/update", RBS_TEST_URL);
            } else {
                $gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", RBS_PROD_URL);
            }
            $gate_url .= substr($this->merchant, 0, -4); // we guess username = login w/o "-api"

            $callback_addresses_string = $this->config->root_url . '/payment/RBSPayment/callback.php?method=callback';
            if ($this->allowCallbacks !== false) {
                $response = $this->_updateGatewayCallback($payment_settings['merchant'], $payment_settings['password'], $gate_url, $callback_addresses_string);

//                if (RBS_ENABLE_LOGGING === true) {
//                    $this->writeLog("[callback_addresses_string]: " . $callback_addresses_string . "\n[RESPONSE]: " . $response);
//                }
            }
        }

        $headers = array(
            'CMS: SimplaCMS version: ' . $this->config->version,
            'Module-Version: ' . $this->module_version,
        );
        $response = $this->_sendGatewayData(http_build_query($args, '', '&'), $action_adr, $headers);

        $response = json_decode($response, true);

        if ($response['errorCode'] != 0) {
            print($response['errorMessage']);
        } else {
            print "REDIRECT";
            header('Location: ' . $response['formUrl']);
        }
        exit;
    }

    public function _updateGatewayCallback($login, $password, $action_address, $callback_addresses_string)
    {
        $headers = array(
            'Content-Type:application/json',
            'Authorization: Basic ' . base64_encode($login . ":" . $password)
        );
        $data['callbacks_enabled'] = true;
        $data['callback_addresses'] = $callback_addresses_string;
        $data['callback_http_method'] = "GET";
        $data['callback_operations'] = "deposited,approved,declinedByTimeout";

        $response = $this->_sendGatewayData(json_encode($data), $action_address, $headers);
        return $response;
    }

    public function _sendGatewayData($data, $action_address, $headers = array())
    {
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_VERBOSE => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $action_address,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $data,
            //                CURLOPT_ENCODING, "gzip",
        ));
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }
}